/**************************************************************************//**
 *
 * Copyright 1998-2012 NetBurner, Inc.  ALL RIGHTS RESERVED
 *   Permission is hereby granted to purchasers of NetBurner Hardware
 *   to use or modify this computer program for any use as long as the
 *   resultant program is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or it's derivatives in part or
 *   in whole are granted.
 *
 *   It may be possible to license this or other NetBurner software for
 *   use on non-NetBurner Hardware.
 *   Please contact sales@Netburner.com for more information.
 *
 *   NetBurner makes no representation or warranties
 *   with respect to the performance of this computer program, and
 *   specifically disclaims any responsibility for any damages,
 *   special or consequential, connected with the use of this program.
 *
 *---------------------------------------------------------------------
 * NetBurner, Inc.
 * 5405 Morehouse Drive
 * San Diego, California 92121
 *
 * information available at:  http://www.netburner.com
 * E-Mail info@netburner.com
 *
 * Support is available: E-Mail support@netburner.com
 *
 *****************************************************************************/

#include "predef.h"
#include <basictypes.h>
#include "constants.h"
#include <system.h>
#include <ucos.h>
#include <ucosmcfc.h>
#include <cfinter.h>
#include <sim.h>
#include <dspi.h>

#include <sim5441x.h>
#include <intcdefs.h>

#define __DEBUG_DSPI
#ifdef __DEBUG_DSPI
#include <stdio.h>
#define DBPRINT_DSPI(args...) iprintf(args)
#else
#define DBPRINT_DSPI(args...)
#endif

extern unsigned long CPU_CLOCK;

// All current Netburner boards have 4 modules, but just in case...
dspiDriverStruct driverCxt[DSPI_MODULE_COUNT] = {
    { NULL, NULL, 0, 0, 0, NULL, DSPI_OK, 0, 0, TRUE }, 
#if DSPI_MODULE_COUNT >=2
    { NULL, NULL, 0, 0, 0, NULL, DSPI_OK, 0, 0, TRUE }, 
#if DSPI_MODULE_COUNT >=3
    { NULL, NULL, 0, 0, 0, NULL, DSPI_OK, 0, 0, TRUE }, 
#if DSPI_MODULE_COUNT >=4
    { NULL, NULL, 0, 0, 0, NULL, DSPI_OK, 0, 0, TRUE }, 
#if DSPI_MODULE_COUNT >=5
    { NULL, NULL, 0, 0, 0, NULL, DSPI_OK, 0, 0, TRUE }, 
#if DSPI_MODULE_COUNT >=6
    { NULL, NULL, 0, 0, 0, NULL, DSPI_OK, 0, 0, TRUE }, 
#if DSPI_MODULE_COUNT >=7
    { NULL, NULL, 0, 0, 0, NULL, DSPI_OK, 0, 0, TRUE }, 
#if DSPI_MODULE_COUNT >=8
    { NULL, NULL, 0, 0, 0, NULL, DSPI_OK, 0, 0, TRUE }, 
#endif
#endif
#endif
#endif
#endif
#endif
#endif
};

typedef struct {
    uint32_t prescale;
    uint32_t baudrate_scaler;
    bool    doubleBaudRate;
    uint32_t baud;
} baudSettingsStruct;

typedef struct {
    uint8_t pcssck;
    uint8_t cssck;
    uint8_t pasc;
    uint8_t asc;
} delaySettingsStruct;

extern "C" {
    void putleds(BYTE LV);
}

/*-----------------------------------------------------------------------
DSPI interrupt service routine.
Called by hardware when DSPI event occurs
------------------------------------------------------------------------*/
void DSPI_Isr( unsigned int moduleNum, volatile dspistruct *spiModule )
{
    register DWORD i;
    register volatile dspistruct &spi = *spiModule;
    register volatile BYTE* Rxbuf = driverCxt[moduleNum].pDSPIRxbuf;
    register volatile BYTE* Txbuf = driverCxt[moduleNum].pDSPITxbuf;
    register DWORD regWordsToWrite = driverCxt[moduleNum].WordsToWrite;
    register DWORD regLastWordsToWrite = driverCxt[moduleNum].LastWordsToWrite;

    if(!(spi.sr & SR_EOQF_MASK))
    {
        driverCxt[moduleNum].DSPI_INT_STATUS = DSPI_BUSY;
        regLastWordsToWrite = 0;
        driverCxt[moduleNum].DSPIfinished = FALSE;
    }

    if(driverCxt[moduleNum].BitsPerQueue <= 8)
    {
        // the FIFOs have 16 slots, so we're reading/writing 16 frames each
        // time through here, and each frame uses 1 byte out of our queues
        if(driverCxt[moduleNum].DSPI_SizeLeft > 16)
        {
            driverCxt[moduleNum].DSPI_SizeLeft -= 16;
        }
        else
        {
            regWordsToWrite = driverCxt[moduleNum].DSPI_SizeLeft;
            driverCxt[moduleNum].DSPI_SizeLeft = 0;
        }

        if(Txbuf && regWordsToWrite)
        {  
            // Set Continuous CS enable, use CTAR[0], get CS bits from Command_Mask
            DWORD fifoCmd = 0x80000000 | ((uint32_t)driverCxt[moduleNum].Command_Mask) << 16;
            for(i = 0; i < regWordsToWrite - 1; i++)
            {
                spi.pushr = fifoCmd | *Txbuf++;
            }
            // Set command EOQ bit
            fifoCmd |= PUSHR_EOQ_BIT;
            // if there's no more to transmit, clear the Contiuous CS bit in the command mask
            if (driverCxt[moduleNum].DSPI_SizeLeft == 0)
            {
                fifoCmd &= 0x7FFFFFFF;
            }
            spi.pushr = fifoCmd | *Txbuf++;

        }    
        if(Rxbuf)
        {   
            for( i=0; i < regLastWordsToWrite; i++)
            {
                *Rxbuf++ = spi.popr;
            }  
        }
    }

    else if( ((driverCxt[moduleNum].BitsPerQueue > 8) 
                && (driverCxt[moduleNum].BitsPerQueue < 17)) 
            || (driverCxt[moduleNum].BitsPerQueue == 32) )
    {   
        if(driverCxt[moduleNum].DSPI_SizeLeft > 32)
        {
            driverCxt[moduleNum].DSPI_SizeLeft -= 32;
        }
        else
        {
            regWordsToWrite = (driverCxt[moduleNum].DSPI_SizeLeft / 2);
            driverCxt[moduleNum].DSPI_SizeLeft = 0;
        }

        if(Txbuf && regWordsToWrite)
        {  
            // Set Continuous CS enable, use CTAR[0], get CS bits from Command_Mask
            DWORD fifoCmd = 0x80000000 | ((uint32_t)driverCxt[moduleNum].Command_Mask) << 16;
            for(i = 0; i < regWordsToWrite - 1; i++)
            {
                spi.pushr = fifoCmd | *(PWORD)Txbuf;
                Txbuf += 2;
            }
            // Set command EOQ bit
            fifoCmd |= PUSHR_EOQ_BIT;
            // if there's no more to transmit, clear the Contiuous CS bit in the command mask
            if (driverCxt[moduleNum].DSPI_SizeLeft == 0)
            {
                fifoCmd &= 0x7FFFFFFF;
            }
            spi.pushr = fifoCmd | *(PWORD)Txbuf;
            Txbuf += 2;
        }    
        if(Rxbuf)
        {   
            // read RX FIFO and move it to the RX buffer
            for( i=0; i < regLastWordsToWrite; i++)
            {
                *(PWORD)Rxbuf = spi.popr;
                Rxbuf += 2;
            }
        }     
    }   

    else if((driverCxt[moduleNum].BitsPerQueue > 16) 
            && (driverCxt[moduleNum].BitsPerQueue < 20))
    {
        if(driverCxt[moduleNum].DSPI_SizeLeft > 32)
        {
            driverCxt[moduleNum].DSPI_SizeLeft -= 32;
        }   
        else
        {
            regWordsToWrite = (driverCxt[moduleNum].DSPI_SizeLeft / 2);
            driverCxt[moduleNum].DSPI_SizeLeft = 0;
        }

        if(Txbuf && regWordsToWrite)
        {
            // Set Continuous CS enable, use CTAR[0], get CS bits from Command_Mask
            DWORD fifoCmd = 0x80000000 | ((uint32_t)driverCxt[moduleNum].Command_Mask) << 16;
            for(i = 0; i < regWordsToWrite - 2; i+=2)
            {
                // Queue a TX with the first (BitsPerQueue - 4) bits, using CTAR[0]
                spi.pushr = fifoCmd | (*(PDWORD)Txbuf >> 4);

                // Queue a TX with the final 4 bits, using CTAR[1]
                fifoCmd |= 0x10000000; // set CTAR[1] bit
                spi.pushr = fifoCmd | (*(PDWORD)Txbuf & 0x0F);
                fifoCmd &= ~0x10000000; // clear CTAR[1] bit, returning CTAR[0]
                Txbuf += 4;
            }
            // Queue a TX with the first (BitsPerQueue - 4) bits, using CTAR[0]
            spi.pushr = fifoCmd | (*(PDWORD)Txbuf >> 4);

            // Queue a TX with the final 4 bits, using CTAR[1], adding the EOQ bit
            fifoCmd |= 0x18000000;
            // if there's no more to transmit, clear the Continuous CS bit in the command mask
            if (driverCxt[moduleNum].DSPI_SizeLeft == 0)
            {
                fifoCmd &= 0x7FFFFFFF;
            }
            spi.pushr = fifoCmd | (*(PDWORD)Txbuf & 0x0F);
            Txbuf += 4;
        }
        if(Rxbuf)
        {
            for( i=0; i < regLastWordsToWrite; i+=2)
            {
                *(PDWORD)Rxbuf = (spi.popr << 4);
                *(PDWORD)Rxbuf |= spi.popr;
                Rxbuf += 4;
            }
        }  
    }

    else if((driverCxt[moduleNum].BitsPerQueue > 19) 
            && (driverCxt[moduleNum].BitsPerQueue < 32))
    {
        if(driverCxt[moduleNum].DSPI_SizeLeft > 32)
        {
            driverCxt[moduleNum].DSPI_SizeLeft -= 32;
        }
        else
        {
            regWordsToWrite = (driverCxt[moduleNum].DSPI_SizeLeft / 2);
            driverCxt[moduleNum].DSPI_SizeLeft = 0;
        }   

        if(Txbuf && regWordsToWrite)
        {  
            // Set Continuous CS enable, use CTAR[0], get CS bits from Command_Mask
            DWORD fifoCmd = 0x80000000 | ((uint32_t)driverCxt[moduleNum].Command_Mask) << 16;
            for(i = 0; i < regWordsToWrite - 1; i++)
            {
                // Queue a TX with the first (BitsPerQueue - 16) bits, using CTAR[0]
                spi.pushr = fifoCmd | *(PWORD)Txbuf;
                Txbuf += 2;

                // Set or clear CTAR[1] <-> CTAR[0]
                fifoCmd ^= 0x10000000;
            }

            // Queue a TX with the final 16 bits, using CTAR[1], adding the EOQ bit
            fifoCmd |= 0x98000000;
            // if there's no more to transmit, clear the Continuous CS bit in the command mask
            if (driverCxt[moduleNum].DSPI_SizeLeft == 0)
            {
                fifoCmd &= ~((uint32_t)driverCxt[moduleNum].Command_Mask) << 16;
            }
            spi.pushr = fifoCmd | ((*(PWORD)Txbuf));
            Txbuf += 2;
        }
        if(Rxbuf)
        {    
            for( i=0; i < regLastWordsToWrite; i++)
            {
                *(PWORD)Rxbuf = spi.popr;
                Rxbuf += 2;
            }
        }
    }

    // Update the spiRecord buffer pointers
    driverCxt[moduleNum].pDSPIRxbuf = Rxbuf;
    driverCxt[moduleNum].pDSPITxbuf = Txbuf;
    
    putleds(driverCxt[moduleNum].DSPIfinished);

    if(driverCxt[moduleNum].DSPIfinished)
    { 
        // do not clear the EOQ flag until *AFTER* setting the halt bit
        // see MCF54418 manual section 40.4.1: Start and stop of DSPI transfers
        spi.mcr |= MCR_HALT_BIT;
        spi.sr &= SR_CLR_FLAGS;
        driverCxt[moduleNum].DSPI_INT_STATUS = DSPI_OK;
        if(driverCxt[moduleNum].DSPI_Sem)
            OSSemPost( driverCxt[moduleNum].DSPI_Sem );

        // Stop the module
        return;
    }

    if(driverCxt[moduleNum].DSPI_SizeLeft == 0)
        driverCxt[moduleNum].DSPIfinished = TRUE;
    driverCxt[moduleNum].WordsToWrite = regWordsToWrite;
    driverCxt[moduleNum].LastWordsToWrite = regWordsToWrite;
    // Writing to the EOQ bit clears it, and starts the SPI transfers.
    spi.mcr &= ~MCR_HALT_BIT;
    spi.sr |= SR_EOQF_MASK;
    return;      
}

INTERRUPT( dspi0_int_routine, 0x2200 )
{
   DSPI_Isr(0, &sim2.dspi0);
}

INTERRUPT( dspi1_int_routine, 0x2200 )
{
   DSPI_Isr(1, &sim2.dspi1);
}

INTERRUPT( dspi2_int_routine, 0x2200 )
{
   DSPI_Isr(2, &sim1.dspi2);
}

INTERRUPT( dspi3_int_routine, 0x2200 )
{
   DSPI_Isr(3, &sim1.dspi3);
}

/*----------------------------------------------------------------------------
BYTE DSPIStart( PBYTE pTxbuf, volatile BYTE* pRxbuf, DWORD num, BYTE SPIModule, 
            OS_SEM* DSPI_Finished_Sem = NULL );

If configured for 8 bits per transfer then the data must be BYTE aligned
If configured for > than 8 bits per transfer then the data must be WORD aligned
If configured for > than 16 bits per transfer then the data must be DWORD aligned
If either RX or TX pointer is assigned 'null' then that comm direction will not occur 
Sends 'num' BYTES  of data starting at PBYTE 'pTxbuf'
Reads and stores in RAM at PBYTE 'pRxbuf'
If DSPI_Finished points to a semaphore then DSPI will POST it when finished
This semaphore is not required but it is the most efficient way to detemine when finished

SPIModule selects which dspi module you wish to use

Returns the current state of the DSPI bus
*///////////////////////////////////////////////////////////////////////////////////
BYTE DSPIStart( BYTE SPIModule, PBYTE pTXBuf, volatile BYTE* pRXBuf, DWORD num, 
            OS_SEM* DSPI_Finished_Sem)
{
    if (SPIModule < 1 || SPIModule > DSPI_MODULE_COUNT - 1)
    {
        return DSPI_BUSY;
    }
    if(driverCxt[SPIModule].DSPI_INT_STATUS != DSPI_OK )         
    {
        return driverCxt[SPIModule].DSPI_INT_STATUS;                   
    }
    if(num == 0)
    {
        if(DSPI_Finished_Sem)
            OSSemPost(DSPI_Finished_Sem);
        return DSPI_OK;
    }
    driverCxt[SPIModule].pDSPIRxbuf = pRXBuf;
    driverCxt[SPIModule].pDSPITxbuf = pTXBuf;
    driverCxt[SPIModule].DSPI_SizeLeft = num;
    driverCxt[SPIModule].DSPI_Sem = DSPI_Finished_Sem;
    driverCxt[SPIModule].WordsToWrite = 16;

    volatile dspistruct *spi = NULL;
    switch (SPIModule)
    {
        case 0:
            spi = &sim2.dspi0;
            break;
        case 1:
            spi = &sim2.dspi1;
            break;
        case 2:
            spi = &sim1.dspi2;
            break;
        case 3:
            spi = &sim1.dspi3;
            break;
    }
    spi->rser = RSER_EOQF_IRQ_ONLY; 

    DSPI_Isr(SPIModule, spi);
    return DSPI_OK;
}
/* 
 *******************************************************************************
 * 
 *  static inline baudSettingsStruct calcBaudSettings( DWORD Baudrate )
 *
 *  This function finds the best settings for the timing register to best 
 *  approximate the desired baudrate. It performs a brute force search and returns
 *  a structure that contains the register setting values that produce the optimal
 *  baudrate, as well as the baudrate that the settings will create.
 *
 *      Parameters:
 *          -Baudrate is the desired baudrate to set
 *
 *      Returns:
 *          baudSettingsStruct containing optimal baudrate settings
 *
 *******************************************************************************
 */
static inline baudSettingsStruct calcBaudSettings( DWORD Baudrate )
{
    DWORD Baud;
    int32_t devMin = 0x80000000;
    int32_t devCur = 0x80000000;

    baudSettingsStruct current;
    baudSettingsStruct best;
    // Prescaler bits for baud rate generator
    // bits value:  0   1   2   3
    // Prescale:    2   3   5   7
    current.prescale = 0; 
    best.prescale = 0;
    // Baud Rate scaler bits
    // bits value:  0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
    // Scaler:      2   4   6   8   16  2^5 2^6 2^7 2^8 2^9 2^A 2^B 2^C 2^D 2^E 2^F    
    current.baudrate_scaler = 0;
    best.baudrate_scaler = 0;
    // Double Baud rate bit
    // Doubles the effective baudrate of module
    current.doubleBaudRate = false;
    best.doubleBaudRate = false;
    for (int j = 0; j < 2 && devMin < 0; j++, current.doubleBaudRate = true)
    {
        for (current.prescale = 0; current.prescale < 4 && devMin < 0; current.prescale++)
        {
            // the prescale values are slighly strange. Get the right one...
            uint32_t prescaleValue = (current.prescale) ? 2 : (1 + 2 * current.prescale);

            // Baud = CPU_CLOCK / 2 * (1 + doubleBaudRate) 
            //          / ( 2 * prescale_value * baudrate_scaler_value)
            //  See 40.4.3.1 of Freescale reference manual (v4) for the MCF54418

            // Baud Rate prescaler value is 2
            Baud = CPU_CLOCK * (1 + current.doubleBaudRate) 
                / (8 * prescaleValue); // Baud Rate prescaler * cpu_clock PS (2) * sysclock PS (always 2)
            if (Baud <= CPU_CLOCK / 3)
            {
                devCur = Baud - Baudrate;
                if (devCur > devMin && devCur <= 0)
                {
                    devMin = devCur;
                    best.prescale = current.prescale;
                    best.baudrate_scaler = 0;
                    best.doubleBaudRate = current.doubleBaudRate;
                    best.baud = Baud;
                    if (devMin == 0) break;
                }
            }

            // Baud Rate prescaler value is 4
            Baud = CPU_CLOCK * (1 + current.doubleBaudRate) 
                / (16 * prescaleValue); // Baud Rate prescaler * cpu_clock PS (2) * sysclock PS (always 2)
            if (Baud <= CPU_CLOCK / 3)
            {
                devCur = Baud - Baudrate;
                if (devCur > devMin && devCur <= 0)
                {
                    devMin = devCur;
                    best.prescale = current.prescale;
                    best.baudrate_scaler = 1;
                    best.doubleBaudRate = current.doubleBaudRate;
                    best.baud = Baud;
                    if (devMin == 0) break;
                }
            }
            
            // Baud Rate prescaler value is 6
            Baud = CPU_CLOCK * (1 + current.doubleBaudRate) 
                / (24 * prescaleValue); // Baud Rate prescaler *  cpu_clock PS (2) * sysclock PS (always 2)
            if (Baud <= CPU_CLOCK / 3)
            {
                devCur = Baud - Baudrate;
                if (devCur > devMin && devCur <= 0)
                {
                    devMin = devCur;
                    best.prescale = current.prescale;
                    best.baudrate_scaler = 2;
                    best.doubleBaudRate = current.doubleBaudRate;
                    best.baud = Baud;
                    if (devMin == 0) break;
                }
            }

            // Now that we've dealt with the non patterned low value prescalers, 
            // we can loop
            for (current.baudrate_scaler = 3; 
                    current.baudrate_scaler <= 0xF && devMin < 0; 
                    current.baudrate_scaler++)
            {
                // clock prescaler value is 2
                Baud = CPU_CLOCK * (1 + current.doubleBaudRate) 
                        / ( 4 * prescaleValue * (0x00000001 << current.baudrate_scaler));
                if (Baud <= CPU_CLOCK / 3)
                {
                    devCur = Baud - Baudrate;
                    if (devCur > devMin && devCur <= 0)
                    {
                        devMin = devCur;
                        best.prescale = current.prescale;
                        best.baudrate_scaler = current.baudrate_scaler;
                        best.doubleBaudRate = current.doubleBaudRate;
                        best.baud = Baud;
                    }
                }
            }
        }
    }
    return best;
}

/*******************************************************************************
 *
 *  static inline 
 *      delaySettingsStruct calcDelaySettings( BYTE QCD, BYTE DTL, 
 *                                              const &baudSettingsStruct baudSet )
 *
 *  In short, this finds the hardware settings that best approximate the requested
 *  delay time.
 *
 *  This function does a brute force search on all possible delay times for the 
 *  post CS/pre SCK delay and the After SCK/pre !CS delay, to get the minimum 
 *  delay that is at least as large as the requested QCD and DTL values.
 *
 *      Parameters:
 *          -QCD is a value in the QDLYR register and will change the delay 
 *          between the assertion of the chip select and the start of the DSPI 
 *          clock.  Default setting of one half DSPI clk will be used if 
 *          parameter is specified as 0x0 or not included.
 *          -DTL is a value in the QDLYR register and will change the delay 
 *          following a tranfer of a single WORD in the DSPI queue.  Default 
 *          reset value of 17/(fsys/2) will be used if parameter is specified 
 *          as 0x0 or not included.
 *          -baudSet is a baudSettings structure containing the baud settings for
 *          the DSPI_CTAR register that the delays are being calculated for.
 *      
 *      Returns:
 *          delaySettingsStruct containing optimal delay setting values
 *
 *******************************************************************************
 */
static inline 
    delaySettingsStruct calcDelaySettings( DWORD QCD, DWORD DTL, const baudSettingsStruct &baudSet )
{
    delaySettingsStruct curr, best = {0, 0, 0, 0};
    int32_t devMin = 0x7FFFFFFF;
    int32_t devCur;

    int32_t cycles;
    // if no QCD is passed, set it for one half the SCK period
    if (!QCD)
    {
        // Delay cycles = cpu frequency / transmit frequency / module clock scaling.
        QCD = CPU_CLOCK / baudSet.baud / 4;
        // was integer division; was there remainder? if so increment the delay cycles
        QCD += (CPU_CLOCK % baudSet.baud == 0) ? 0 : 1; 
    }
    
    // if they try to set a value we should get as close as possible
    // PCSSCK prescaler settings go from 0 -> 3, see table 40.6 of Freescale manual
    for (curr.pcssck = 0; curr.pcssck < 4 && devMin > 0; curr.pcssck++)
    {
        // CSSCK settings go from 0 -> 0xFF, but once they're above 8 
        // the scaler is greater than 256
        for (curr.cssck = 0; curr.cssck < 8 && devMin > 0; curr.cssck++)
        {
            cycles = (1 + 2 * curr.pcssck) * (0x00000002 << curr.cssck);
            devCur = cycles - QCD;
            if (devCur < devMin && devCur >=0)
            {
                devMin = devCur;
                best.pcssck = curr.pcssck;
                best.cssck = curr.cssck;
            }
        }
    }

    // Attempting to keep same behavior as advertised by DSPI on the other modules.
    // Advertised as 17/f_sys/2 cycles delay
    if (!DTL) DTL = 17;

    devMin = 0x7FFFFFFF;
    // if they try to set a value we should get as close as possible
    // PCSSCK prescaler settings go from 0 -> 3, see table 40.6 of Freescale manual
    for (curr.pasc = 0; curr.pasc < 4 && devMin > 0; curr.pasc++)
    {
        // CSSCK settings go from 0 -> 0xFF, but once they're above 8 
        // the scaler is greater than 256
        for (curr.asc = 0; curr.asc < 16 && devMin > 0; curr.asc++)
        {
            cycles = (1 + 2 * curr.pasc) * (0x00000002 << curr.asc);
            devCur = cycles - DTL;
            if (devCur < devMin && devCur >=0)
            {
                devMin = devCur;
                best.pasc = curr.pasc;
                best.asc = curr.asc;
            }
        }
    }

    return best;
}

/*----------------------------------------------------------------------------
BYTE DSPIInit( BYTE SPIModule = DEFAULT_SPI_MODULE, DWORD Baudrate = 2000000, 
               BYTE QueueBitSize = 0x8, BYTE CS = 0xF,
               BYTE CSPol = 0x1, BYTE ClkPolarity = 0x0, BYTE ClkPhase = 0x1, 
               BOOL DoutHiz = TRUE, BYTE QCD = 0x0, BYTE DTL = 0x0 );  

DSPI initialization routine
-SPIModule selects which dspi module you wish to use
-Baudrate which you wish to be the max transfer speed.  Will pick next lowest speed if not available
-QueueBitSize is the size of each queue to be transfered.  If the number of bits per 
transfer is greater then 8 then the data must be word alligned.
-CS is the the a value between 0 and 15 and determines which DSPI chip selects are asserted
durring a DSPI transfer.  An active chip select is represented with a 0 bit.
-CSPol is a value that states the polarity of a chip select when it is inactive.
-ClkPolarity should be set to either '0' or '1' and this will be the logic level
of the clock when inactive.
-ClkPhase should be set to either '0' if data is captured on the leading clock edge or '1' if 
data is changed on the leading clock edge.
-DoutHiz should be true if the DOUT line needs to be in high impedance in between transfers.
-QCD is a value in the QDLYR register and will change the delay between the assertion of 
the chip select and the start of the DSPI clock.  Default setting of one half DSPI clk
will be used if parameter is specified as 0x0 or not included.
-DTL is a value in the QDLYR register and will change the delay following a tranfer of 
a single WORD in the DSPI queue.  Default reset value of 17/(fsys/2) will be used if 
parameter is specified as 0x0 or not included.

Returns the current state of the DSPI bus
*///////////////////////////////////////////////////////////////////////////////////
BYTE DSPIInit( BYTE SPIModule, DWORD Baudrate, BYTE QueueBitSize, BYTE CS, BYTE CSPol, 
               BYTE ClkPolarity, BYTE ClkPhase, BOOL DoutHiz, BYTE QCD, BYTE DTL )
{
    register int i;
    
    // DSPI0 is not brought out on any board
    if (SPIModule < 1 || SPIModule > DSPI_MODULE_COUNT - 1)
    {
        return DSPI_BUSY;
    }

    volatile dspistruct *spi = NULL;
    
    switch (SPIModule)
    {
        case 0:
            spi = &sim2.dspi0;
            break;
        case 1:
            spi = &sim2.dspi1;
            break;
        case 2: 
            spi = &sim1.dspi2;
            break;
        case 3: 
            spi = &sim1.dspi3;
            break;
    }
    if (!spi)
    {
        return driverCxt[SPIModule].DSPI_INT_STATUS;
    }

    if(driverCxt[SPIModule].DSPI_INT_STATUS != DSPI_OK ) 
    {
        return driverCxt[SPIModule].DSPI_INT_STATUS;                   
    }

    driverCxt[SPIModule].BitsPerQueue = QueueBitSize;
    driverCxt[SPIModule].Command_Mask = CS;

    DBPRINT_DSPI("BitsPerQueue: %d\r\n", driverCxt[SPIModule].BitsPerQueue);
    // The DSPI engine cannot handle continuous SCK 
    // or modified timing at > CPU_CLOCK/3
    if (Baudrate > CPU_CLOCK/3) Baudrate = CPU_CLOCK/3 - CPU_CLOCK/30;
    
    baudSettingsStruct baudSettings = calcBaudSettings( Baudrate );
    // see 23.2.3 of MCF5270 manual for explanation of DTL * 32
    delaySettingsStruct delaySettings = calcDelaySettings( QCD, (DWORD)DTL * 32, baudSettings );

    // If the baud we're going to transmit at is above certain values, 
    // we need to adjust our slew rates to prevent signal integrity issues
    if (baudSettings.baud > 40000000)
    {
        switch (SPIModule)
        {
            case 0:
                // DSPI0 slewrate is bits [1:0], onewire is bits [5:4]
                sim1.gpio.srcr_dspiow   &= ~0x03;
                sim1.gpio.srcr_dspiow   |= 0x02;
                break;
            case 1:
                // SDHC slewrate is bits [1:0], others unused
                sim1.gpio.srcr_sdhc     &= ~0x03;
                sim1.gpio.srcr_sdhc     |= 0x02;
                break;
            case 2:
                // DSPI2 pins are UART0, UART0 slewrate is bits [1:0]
                sim1.gpio.srcr_uart     &= ~0x03;
                sim1.gpio.srcr_uart     |= 0x02;
                break;
            case 3:
                // DSPI3 pins are UART1, UART1 slewrate is bits [3:2]
                sim1.gpio.srcr_uart     &= ~0x0D;
                sim1.gpio.srcr_uart     |= 0x08;
                break;
        }
    }
    else if (baudSettings.baud > 4000000)
    {
        switch (SPIModule)
        {
            case 0:
                // DSPI0 slewrate is bits [1:0], onewire is bits [5:4]
                sim1.gpio.srcr_dspiow   &= ~0x03;
                sim1.gpio.srcr_dspiow   |= 0x01;
                break;
            case 1:
                // SDHC slewrate is bits [1:0], others unused
                sim1.gpio.srcr_sdhc     &= ~0x03;
                sim1.gpio.srcr_sdhc     |= 0x01;
                break;
            case 2:
                // DSPI2 pins are UART0, UART0 slewrate is bits [1:0]
                sim1.gpio.srcr_uart     &= ~0x03;
                sim1.gpio.srcr_uart     |= 0x01;
                break;
            case 3:
                // DSPI3 pins are UART1, UART1 slewrate is bits [3:2]
                sim1.gpio.srcr_uart     &= ~0x0D;
                sim1.gpio.srcr_uart     |= 0x04;
                break;
        }

    }
    
    DBPRINT_DSPI("Desired Baudrate: %lu\r\n", Baudrate);
    DBPRINT_DSPI("Baud Rate and settings:\r\n");
    DBPRINT_DSPI("          Baud:  %lu\r\n", baudSettings.baud);
    DBPRINT_DSPI("      prescale:  %lu\r\n", baudSettings.prescale);
    DBPRINT_DSPI("    BRprescale:  %lu\r\n", baudSettings.baudrate_scaler);
    DBPRINT_DSPI("      doubleBR:  %s\r\n\r\n", baudSettings.doubleBaudRate ? "TRUE" : "FALSE");

    DBPRINT_DSPI("Delay settings:\r\n");
    DBPRINT_DSPI("    pcssck: %u\r\n", delaySettings.pcssck);
    DBPRINT_DSPI("     cssck: %u\r\n", delaySettings.cssck);
    DBPRINT_DSPI("      pasc: %u\r\n", delaySettings.pasc);
    DBPRINT_DSPI("       asc: %u\r\n", delaySettings.asc);

    // Begin constructing the Baud rate and delay value mask for the timing registers
    i = 0x00000000;
    i |= ( (DWORD) baudSettings.doubleBaudRate ) << 31;
    i |= baudSettings.prescale << 16;
    i |= baudSettings.baudrate_scaler;
    i |= ( (DWORD) delaySettings.pcssck )   << 22;
    i |= ( (DWORD) delaySettings.cssck )    << 12;
    i |= ( (DWORD) delaySettings.pasc )     << 20;
    i |= ( (DWORD) delaySettings.asc )      <<  8;
    if(ClkPolarity)
        i |= CTAR_CLOCK_POLARITY;
    if(ClkPhase)
        i |= CTAR_CLOCK_PHASE;   
    
    // Set base values for timing registers.
    spi->ctar[0] = i;
    spi->ctar[1] = i;

    // If the bit size is less that 17, we can send it in a single frame
    if((QueueBitSize >= 4) && (QueueBitSize < 17))
    {
        spi->ctar[0] |= QueueBitSize - 1 << 27; // Frame size starts at bit 27
    }
    // Minimum frame size is 4 bits, therefore if the bit size is less than 20
    // we need to calculate the length of the frames a little differently
    else if((QueueBitSize > 16) && (QueueBitSize < 20))
    {
        spi->ctar[0] |= QueueBitSize - 5 << 27; // Frame size starts at bit 27
        spi->ctar[1] |= CTAR_FRAME_4BIT;
    }
    else if ((QueueBitSize > 19) && (QueueBitSize < 32))
    {
        // Transmit 16 bits in second chunk, and the variable ammount in the first.
        spi->ctar[0] |= QueueBitSize - 17 << 27; // Frame size starts at bit 27
        spi->ctar[1] |= CTAR_FRAME_16BIT;
    }
    else if(QueueBitSize == 32)
    {
        spi->ctar[0] |= CTAR_FRAME_16BIT;
    }

    spi->mcr = MCR_MASTER_INIT | (((DWORD) CSPol) << 16); // Chip Select inactive state bits start at bit 16

    switch (SPIModule)
    {
        case 0:
            SETUP_DSPI0_ISR(&dspi0_int_routine, 2);
            break;
        case 1:
            SETUP_DSPI1_ISR(&dspi1_int_routine, 2);
            break;
        case 2:
            SETUP_DSPI2_ISR(&dspi2_int_routine, 2);
            break;
        case 3:
            SETUP_DSPI3_ISR(&dspi3_int_routine, 2);
            break;
    }

    return driverCxt[SPIModule].DSPI_INT_STATUS;
}

/*----------------------------------------------------------------------------
BOOL DSPIdone();

returns TRUE if DSPI is finished
returns FALSE if DSPI is active
*///////////////////////////////////////////////////////////////////////////////////
BOOL DSPIdone( BYTE SPIModule )
{
   return driverCxt[SPIModule].DSPIfinished;
}
       

